/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 0.80B */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "abimport.h"
#include "globals.h"
#include "proto.h"

int
screen_setup( PtWidget_t *link_instance, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	link_instance = link_instance, apinfo = apinfo, cbinfo = cbinfo;

	return( Pt_CONTINUE );

	}

int
screen_select( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	PhPointerEvent_t	*pdata;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	pdata = PhGetData( cbinfo->event );	

	if ( pdata->buttons == 1 )
		screen_prev( widget, apinfo, cbinfo );
	else
		screen_next( widget, apinfo, cbinfo );

	return( Pt_CONTINUE );

	}

int
screen_prev( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/* haven't started yet... */
	if ( slide_curr == -1 )
		return( Pt_CONTINUE );

	/* get previous slide */
	if ( slide_curr != 0 ) {
		if ( scroll_inc != 768 ) {
			screen_down( );
			ApClearWidget( ABW_spane );
			}
		screen_update( ABW_spane, --slide_curr, 1 );
		if ( scroll_inc != 768 )
			screen_up( );
		}

	/* read previous slide */
	if ( slide_curr != 0 )
		screen_read( slide_curr - 1 );

	/* free slide 2 ahead */
	if ( slide_curr < slide_ctr - 2 ) 
		screen_free( slide_curr + 2 );

	return( Pt_CONTINUE );

	}

int
screen_next( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/* display next slide */
	if ( slide_curr < slide_ctr - 1 ) {
		if ( scroll_inc != 768 ) {
			screen_down( );
			ApClearWidget( ABW_spane );
			}
		screen_update( ABW_spane, ++slide_curr, 1 );
		if ( scroll_inc != 768 )
			screen_up( );
		}

	/* read next slide */
	if ( slide_curr != slide_ctr - 1 )
		screen_read( slide_curr + 1 );

	/* free slide 2 back */
	if ( slide_curr > 1 )
		screen_free( slide_curr - 2 );

	return( Pt_CONTINUE );

	}

void
screen_down( )

	{
	int			i;
	PtArg_t		args[1];

	for ( i = 0; i <= 768; i += scroll_inc ) {
		PtSetArg( &args[0], Pt_ARG_SCROLL_AREA_POS_Y, i, 0 );
		PtSetResources( ABW_sscreen, 1, args );
		PtUpdate( );
		}

	}

void
screen_up( )

	{
	int			i;
	PtArg_t		args[1];

	PtFlush( );
	for ( i = 768; i >= 0; i -= scroll_inc ) {
		PtSetArg( &args[0], Pt_ARG_SCROLL_AREA_POS_Y, i, 0 );
		PtSetResources( ABW_sscreen, 1, args );
		PtUpdate( );
		}

	}

void
screen_update( PtWidget_t *parent, int slide_no, int palette )

	{
	PhArea_t	area;
	PtArg_t		args[10];
	PhImage_t	*i;

	/* not ready */
	if ( slide_no == -1 )
		return;
	
	/* check for palette file */
	if ( palette )
		palette_update( pals[slide_no], images[slide_no] );

	if ( scroll_inc == 768 ) {
		/* establish new graphic context */
		old_gc = PgSetGC( gc );
		PgSetRegion( PtWidgetRid( parent ) );

		/* make life easier */
		i = images[slide_no];

		/* copy slide to shared imaged */
		if ( slide_shared )
			memcpy( sh_image, i->image, i->size.h * i->size.w );

		/* draw image */
		area.pos.x = area.pos.y = 0;
		PgSetPalette( i->palette, 0, 0, i->colors, Pg_PALSET_SOFT, 0 );
		if ( slide_shared )
			PgDrawImagemx( sh_image, i->type, &area.pos, (PhPoint_t *)&i->size, i->bpl, 0 );
		else
			PgDrawImagemx( i->image, i->type, &area.pos, (PhPoint_t *)&i->size, i->bpl, 0 );

		/* reset old graphic context */
		PgFlush( );
		PgSetGC( old_gc );
		}
	else {
		/* set dimensions */
		area.pos.x = area.pos.y = 0;
		area.size.w = 1024;
		area.size.h = 768;
		PtSetArg( &args[0], Pt_ARG_AREA, &area, 0 );
		PtSetArg( &args[1], Pt_ARG_LABEL_TYPE, Pt_IMAGE, Pt_IMAGE );
		PtSetArg( &args[2], Pt_ARG_LABEL_DATA, images[slide_no], sizeof( PhImage_t ) );
		PtRealizeWidget( PtCreateWidget( PtLabel, parent, 3, args ) );
		}

	}

void
screen_read( int slide_no )

	{
	char		path[PATH_MAX];
	PxMethods_t methods;

	/* resolve path offset */
	if ( slides[slide_no][0] == '/' )
		strcpy( path, slides[slide_no] );
	else {
		strcpy( path, slide_path );
		strcat( path, "/" );
		strcat( path, slides[slide_no] );
		}
	
	memset( &methods, 0, sizeof( PxMethods_t ) );
	methods.flags |= PX_LOAD;
	if ( ( images[slide_no] = PxLoadImage( path, &methods ) ) == NULL )
		return;

	}

void
screen_free( int slide_no )

	{

	if ( !images[slide_no] )
		return;

	if ( images[slide_no]->palette ) {
		free( images[slide_no]->palette );
		images[slide_no]->palette = NULL;
		}

	if ( images[slide_no]->image ) {
		free( images[slide_no]->image );
		images[slide_no]->image = NULL;
		}

	free( images[slide_no] );
	images[slide_no] = NULL;

	}


